use crate::config::config::Config;
use crate::store::{ignorer::Ignorer, nsobject::NsObject};
use std::fs;
// use glob::glob;
use std::path::{Path, PathBuf};

pub struct AddArgs {
    /// List of files/dirs to add
    pub files: Vec<String>,
    /// Allow ignored files
    pub force: bool,
    /// Select all files
    pub all: bool,
}

pub fn exec(args: AddArgs, config: Config) {
    // Init ignorer
    let mut ignorer = Ignorer::new(config.get_root_unsafe());
    ignorer.active_nsignore(!args.force);

    if args.all {
        return add_dir(config.get_root_unsafe(), &mut ignorer);
    }

    for obj_to_add in args.files.iter() {
        let mut path_to_add = config.execution_path.clone();
        path_to_add.push(PathBuf::from(obj_to_add));

        if path_to_add.exists() {
            if path_to_add.is_dir() {
                add_dir(&path_to_add, &mut ignorer);
            } else {
                add_to_index(&path_to_add);
            }
        } else if NsObject::from_local_path(&path_to_add).exists() {
            add_to_index(&path_to_add);
        } else {
            // try globbing
            // todo!()
            // else panic
            panic!("fatal: pathspec '{}' did not match any files", obj_to_add);
        }
    }
    /*
       for all files
           if globbing
               take all match glob nextsyncignore
           else
               if dir
                   add dir
                   add all childs with nextsyncignore
               else
                   add files
    */
}

/// Add all files in a directory
///
/// # Parameters
/// * `dir`: the directory to add from
/// * `force`: true if we should apply the nextsyncignore's rules
fn add_dir(dir: &PathBuf, ignorer: &mut Ignorer) {
    let entries = match fs::read_dir(dir) {
        Ok(entries) => entries,
        Err(err) => {
            eprintln!("Failed to read {} ({err})", dir.display());
            return;
        }
    };

    // Read locals files and folders
    for entry in entries {
        let entry = match entry {
            Ok(entry) => entry,
            Err(err) => {
                eprintln!("Failed to read entry {err}");
                continue;
            }
        };

        // ignore object if it is ns config file or nsignored
        if ignorer.should_ignore(&entry.path()) {
            continue;
        }

        if entry.path().is_dir() {
            add_dir(&entry.path(), ignorer);
            add_to_index(&entry.path());
        } else {
            add_to_index(&entry.path());
        }
    }
}

fn add_to_index(path: &PathBuf) {
    println!("{}", path.display());
}
